###########################################################################
#
#   Copyright (c) 2000,2001 Morten W. Petersen <morten@esol.no>
#	Copyright (c) 2000 Erik Enge <erik@esol.no>
#	Copyright (c) 2000 Thingamy Ltd.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
#    MA 02111-1307 USA
#
#
###########################################################################

# $Author: morten $
# $Revision: 1.12 $

__doc__ = "Documentation string"
__version__ = "0.1.0"

debug = 1

from Globals import HTMLFile
import ZODB
import string
import types

class base_item:
	"Documentation string"

	meta_types = ()

	__ac_permissions__ = () # !FIXME!

	base_delete = HTMLFile('base_delete', globals())
	base_edit = HTMLFile('base_edit', globals())
	base_edit_display = HTMLFile('base_edit_display', globals())

	color_id0 = 'lightblue'
	color_body0 = 'white'
	color_body1 = 'lightgrey'
	color_action1 = 'lightblue'
	color_action0 = color_action1
	color_header0 = 'lightyellow'
	color_header1 = '#6699cc'
	color_header_description0 = '#abcdef'
	color_sequence_even0 = '#cccccc'
	color_sequence_odd0 = '#aeaeae'
	table_cellpadding0 = 1
	table_cellspacing0 = 1
	table_border0 = 0

	counter = 0

	def local_navigation(self, object, REQUEST):
		"Return navigation local to this class."
		
		if len(self.meta_types) != 0:
			block = '<form action="'+str(REQUEST.URL1)+'/" method="GET">\n'
			block = block + '\t<select name=":method">\n'

			for sub_class in self.meta_types:
				block = block + '\t\t<option value="%s">%s</option>\n' % \
						(sub_class['action'], sub_class['name'])
			block = block + '\t</select>\n\t<input type="submit" value=" Add ">\n</form>\n'

		else:
			
			block = ""
			
		block = block + '<p><a href="../base_delete?ids=%s">Delete</a><br>' % object.id
		block = block + '<a href="base_edit">Edit</a></p>' 
		block = block + '<p><a href="/Control_Panel/Products/%s/Help/' % \
			(self.product_name) +	string.lower(object.meta_type)+'.stx">Help</a></p>'

		return block


        def name_title_or_id(self):
                "Returns either firstname + surname, name, title, meta_type or id - in that order."
                if self.get_property('name') == 1:
                        name=self.name
                else:
                        name=''

                if self.get_property('firstname') == 1:
                        firstname=self.firstname
                        surname=self.surname
                else:
                        firstname=''
                        surname=''

                title=self.title
                if callable(title):
                        title=title()

                id=self.id
                if callable(id):
                        id=id()

                if len(firstname) > 0:
                        return firstname +' '+ surname
                elif len(name) > 0:
                        return name
                elif len(title) > 0:
                        return title 
                else:
                        return id


	def local_navigation(self, *args, **kw):
		"""Returns a set of widgets for adding objects."""

		actions = []

		for subclass in self.meta_types:

			actions.append("""<option value="%s">%s</option>""" % \
						(subclass['action'], subclass['name']))

		return """<form><select name=":method">%s</select><input type="submit" value=" Add "></form>""" % string.join(actions)


	def get_preferences_url(self):
		"""Returns the URL used to manage product instance preferences."""

		return self.relative_url() + '/base_edit'



	def get_property(self, prop_name):
		"Return true if self._properties contains prop_name."
		for x in range(len(self._properties)): 
			for key in self._properties[x].keys():
				if str(prop_name) == str(self._properties[x]['id']):
					return 1
		return 0


	def get_properties(self):
		"Return a list of properties for this object."

		return self._properties


	def copy(self):
		"""Returns a copy of self."""

		return self.getParentNode().manage_copyObjects(self.id)


	def paste(self, copy_data, new_id=None):
		"""Pastes the copy into self."""

		self.manage_pasteObjects(cb_copy_data=copy_data)


	def cut(self):
		"""Cuts self from containing object."""

		return self.getParentNode().manage_cutObjects(self.id)


	def delete(self, RESPONSE=None):
		"""Deletes self."""

		self.getParentNode().manage_delObjects(self.id)

		if RESPONSE is not None:

			RESPONSE.redirect(self.getParentNode().relative_url())


	def base_edit_properties(self, REQUEST):
		"""Update properties, with a nice redirect."""

		# This method assumes that all properties defined
		# are settable by anyone with access to the object.

		# FIXME
		# setattr = setattr

		if REQUEST.has_key('__ignore_keyerror') and REQUEST['__ignore_keyerror']:

			for property in self._properties:

				try:

					exec("self."+str(property['id'])+" = REQUEST."+str(property['id']))

				except KeyError:

					pass

		else:

			for property in self._properties:

				exec("self."+str(property['id'])+" = REQUEST."+str(property['id']))

		return REQUEST.RESPONSE.redirect(self.absolute_url())


	def rem_navigation(self, object, separator_html=' &gt; '):
		""

		path = []

		physical_path = list(self.getPhysicalPath())
		physical_path.reverse()

		for element_index in range(len(physical_path)):

			path.append("""<a href="%s">%s</a>""" % \
						('../' * element_index,
						 physical_path[element_index] or 'base'))


		path.reverse()

		return string.join(path, separator_html)


	def footer(self, context):
		""

		return ""


	def product_installed(self, product_name):
		"""A method that returns a truth value if given
		product is installed."""

		if product_name in self.Control_Panel.Products.objectIds():

			return 1

		else:

			return 0


	def get_nearest_object_by_meta_type(self, meta_type):
		"""Returns the nearest instance with given meta type."""

		instance = None

		reference = self

		while 1:

			if reference == None:

				# If we've come to the end of the line

				break

			instances = \
				reference.objectValues(meta_type)

			if instances:

				# If we found objects on the way

				instance = \
					instances[0]

				break

			reference = reference.getParentNode()

		return instance


	def get_all_objects_by_meta_type(self, meta_type):
		"""Returns all objects in acquisition path of given meta type."""

		instances = []

		reference = self

		while 1:

			instances = instances + reference.objectValues(meta_type)

			reference = self.getParentNode()

			if reference is None:

				break

		return instances


	def sneak_peak(self):
		"""Brings up a useful display of the object."""

		# FIXME -- how do we get the object location in memory?
		# Why does this raise an exception?

		return_string = "<class %s at %s" % (self.__class__.__name__,
							'????????')
							#__builtins__.id(self))

		for property in self._properties:

			return_string = return_string + "%s: %s\n" % (property,
									self[property])

		else:

			return_string = return_string + '\n>'

		return return_string


	def manage_delete(self, ids=[], REQUEST=None, RESPONSE=None):
		"""Deletes given ids."""

		if type(ids) is type(''):

			ids = [ids]

		self.manage_delObjects(ids)

		if REQUEST is not None:

			if REQUEST.has_key('__referrer'):

				RESPONSE.redirect(REQUEST['__referrer'])

			else:

				RESPONSE.redirect(self.absolute_url())


	def redirect_next_id(self, REQUEST, RESPONSE):
		"""Returns the 'next' object id."""

		object_id = REQUEST['__object_id']
		
		if REQUEST.has_key('__meta_type') and REQUEST['__meta_type']:

			meta_type = REQUEST['__meta_type']

		else:

			meta_type = None

		if REQUEST.has_key('__resolve_url') and REQUEST['__resolve_url']:

			reference = REQUEST.resolve_url(REQUEST['__resolve_url'])

		else:

			reference = self

		objectIds = reference.objectIds(spec=meta_type)

		try:

			next = objectIds[objectIds.index(object_id) + 1]

		except IndexError:

			# We've got the last id, wrapping around to the first one.

			next = objectIds[0]

		RESPONSE.redirect(reference.absolute_url() + '/' + next)


	def redirect_previous_id(self, REQUEST, RESPONSE):
		"""Returns the 'previous' object id."""

		object_id = REQUEST['__object_id']

		if REQUEST.has_key('__meta_type') and REQUEST['__meta_type']:

			meta_type = REQUEST['__meta_type']

		else:

			meta_type = None

		if REQUEST.has_key('__resolve_url') and REQUEST['__resolve_url']:

			reference = REQUEST.resolve_url(REQUEST['__resolve_url'])

		else:

			reference = self

		objectIds = reference.objectIds(spec=meta_type)

		try:

			previous = objectIds[objectIds.index(object_id) - 1]

		except ValueError:

			# We've got the first id in the list, wrap it.

			previous = objectIds[-1]

		RESPONSE.redirect(reference.absolute_url() + '/' + previous)


	def export_as_understandable_xml(self):
		"""Exports the instance in a 'pure' xml format."""

		# We want this to export variables defined in the _properties
		# dictionary only.

		# TODO !

		pass		


	def manage_delete_all(self, REQUEST=None, RESPONSE=None):
		"""Method that deletes all messages in folder."""

		if REQUEST is not None:

			if REQUEST.has_key('confirm') and \
				REQUEST['confirm']:

				pass

			else:

				RESPONSE.redirect(self.absolute_url())
				return

		self.manage_delete(ids=self.objectIds())

		RESPONSE.redirect(self.absolute_url())


	def relative_url(self):
		"""Returns the URL relative to the hostname and port."""

		return '/' + self.absolute_url(relative=1)


	def resolve_relative_url(self, relative_url, REQUEST):
		"""Method that resolves a URL assuming it is in the local namespace."""

		return self.restrictedTraverse(path=relative_url)


	def get_parent_node_title(self):
		"""Returns the title of the parent node."""

		return self.getParentNode().title


	def get_parent_node_relative_url(self):
		"""Returns the relative url of the parent node."""

		return self.getParentNode().relative_url()


	# ?FIXME?

	def navigation(self, object, REQUEST, navi_html=""):
		"Return some navigation HTML code."
		url = object.REQUEST.SERVER_URL

		jump_navi = ''

		try:
			for element in object.getPhysicalPath():
				url = url + str(element) + "/"
			navi_html = '<a href="'+url+'">'+str(object.name_title_or_id())+'</a> <b>></b> ' + str(navi_html)
			return self.navigation(object.getParentNode(), REQUEST, navi_html)
		except AttributeError:
			return '<table width="100%" cellpadding="0" cellspacing="0"><tr><td valign="center" align="left" bgcolor="lightyellow" width="90%"><font size="-1">'+navi_html[:-9]+'</font></td><td valign="center" align="center" bgcolor="lightyellow" width="10%">'+jump_navi+'</td></tr></table>'
