def build_filter(args):
	"""Builds a lambda expression we can use for filtering

	args is a sequence-like object, where the args[n][0] is the
	attribute-name of the object, and args[n][1] is the value to
	check against."""

	# Should we maybe include a third index in the sequence that specifies
	# the comparison operator?

	eval_string = ''

	for arg in args:

		eval_string = eval_string + 'a.%s == %s and ' % (arg[0],`arg[1]`)

	else:

		eval_string = eval_string[:-5]

	return eval('lambda a: %s' % eval_string)


def do_filtering(sequence, *args):

	return filter(build_filter(args), sequence)


def pad_integer(integer, size=2):
	"""Function that pads an integer to a string with the given size."""

	string_integer = str(integer)

	string_integer_length = len(string_integer)

	if string_integer_length < size:

		return ('0' * (size - string_integer_length)) + string_integer

	else:

		return string_integer


def format_time(value):
	"""Returns a HH:MM style formatted representation of the time."""

	value = int(value)

	hours = value / (60*60)
	minutes = value % (60*60) / 60

	return pad_integer(hours) + ':' + pad_integer(minutes)


def compile_time(hour, minute):
	"""Returns a compiled integer value."""

	hour = int(hour)
	minute = int(minute)

	return ((hour * 60 * 60) + (minute * 60))


def make_hour_minutes_select(hour=None, minute=None, name=None, seconds=None, minute_step=5,
			selected_hour=None, selected_minute=None, integer=1):
	"""Returns a <SELECT> widget."""

	if selected_minute or selected_hour:

		tmp = make_range_select(name + '_hour', 0, 24, pad=1, selected = selected_hour)
		tmp = tmp + make_range_select(name + '_minute', 0,60, pad=1,
					selected = selected_minute, step=minute_step)

	elif seconds:

		seconds = int(seconds)

		hour = seconds / (60*60)
		minute = (seconds%(60*60)/60)

		if selected_minute is None and \
			selected_hour is None:

			selected_hour = hour
			selected_minute = minute

		tmp = make_range_select(name + '_hour', 0, 24, pad=1, selected = selected_hour)
		tmp = tmp + make_range_select(name + '_minute', 0,60, pad=1,
					selected=selected_minute, step=minute_step)

	else:

		tmp = make_range_select(name + '_hour', 0, 24, pad=1, selected = 12)
		tmp = tmp + make_range_select(name + '_minute', 0,60, pad=1, selected = 0,
					step=minute_step)

	return tmp


def make_range_select(name, start, stop, step=1, selected=None, pad=None, integer=1):
	"""Returns a <SELECT> widget."""

	if integer:

		tmp = """<select name="%s:int">""" % name

	else:

		tmp = """<select name="%s">""" % name

	option_dict = {}

	if selected:

		for i in range(start, stop, step):

			if pad:

				displayed_value = pad_integer(i)

			else:

				displayed_value = i

			if i == selected:

				tmp = tmp + """<option value="%s" selected>%s</option>\n""" % (i, displayed_value)

			else:
				
				tmp = tmp + """<option value="%s">%s</option>\n""" % (i, displayed_value)
			

	else:

		for i in range(start, stop, step):

			if pad:

				displayed_value = pad_integer(i)

			else:

				displayed_value = i

			tmp = tmp + """<option value="%s">%s</option>\n""" % (i, displayed_value)

	tmp = tmp + """</select>"""

	return tmp


def format_size(bytes, details=None):
	"""Renders a human-readable display of the size."""

	return_string = ""

	if bytes > 1024*1024:

		return_string = return_string + \
			"%.2f MB" % (bytes / 1024.0 / 1024)

	else:

		return_string = return_string + \
			"%.2f KB" % (bytes / 1024.0)

	if details:

		return_string = return_string + " (%s bytes)" % bytes

	return return_string	
